/*H+ UIWindow.h
*
*    SCCS ID: @(#)UIWindow.h 1.3 92/05/23 SMI 
*
*    Copyright (c) 1990-1992 Sun Microsystems, Inc.  All rights reserved.
*    See LEGAL_NOTICE file for terms of the license.
*
*    PURPOSE: 
*
*    Defines the UIWindow class.
*
*    AUTHOR: Joe Warzecha and Mark Soloway
*
*    REVISION HISTORY:
*
*    Who       When       Why                   
*    --------- ---------- -----------------------
*    JDZ       04/30/91   Preparation for code review
*
*H-*/

/*C+ UIWindow
*
*    DESCRIPTION:
*
*    It describes such features as whether the window is resizable, whether 
*    its footer displays, the window's height and width and setting a function 
*    that executes when a user attempts to destroy the window.  The Derived 
*    Classes are specific instances of a window, and define how the window 
*    will look and behave, such as whether it is a base window for the entire 
*    application or whether it pops up when a certain action occurs.
*
*    INHERITED CLASSES:
*
*    UIObject - The base class of all the UI classes in the UIC.
*    
*    ORDINARY DATA MEMBERS:
*
*    GenericList	 displayList   - Lists UIDisplay objects that are
*    contained in this window.
*   
*    Boolean		 resizable     - TRUE if window is resizable.
*
*    Boolean	 	 displayFooter - TRUE if the footer should be displayed.
*
*    UICHandlerFunction	 doneHandler   - The function to execute when the user 
*    attempts to unpin or DISMISS a PopUpWindow or QUIT a BaseWindow.
*
*    char               *leftFooter    - Pointer to the left footer.
*
*    char   		*rightFooter   - Pointer to the right footer.
*
*    Boolean             fitHeight     - TRUE if the window should fit to the 
*    height around the Displays it contains in the specified Dimension.
*
*    Boolean             fitWidth      - TRUE if the window should fit to the
*    width around the Displays it contains in the specified Dimension.
*
*    GenericList         childWindows  - Contains list of all children windows.
*
*    static GenericList *peerWindows   - Pointer to the list of all peers 
*    windows.
*
*    UIWindow		*parentWindow  -  Pointer to the parent window.
*
*    Boolean		 useDefaultDoneHandlerFlag - TRUE if the default done
*    handler should be used.
*
*    PUBLIC MEMBER FUNCTIONS:
*
*            void    setResizable (Boolean flag) - Sets whether or not the
*            window is resizable.  Default: PopUpWindow is resizable.
*
*            void    setDisplayFooter (Boolean footer) - Sets whether or not
*            the window's footer should be displayed.  Default: Displayed.
*
*            void    setLeftFooter (char *footer) - Specifies the string to
*            be displayed as the window's left footer.
*
*            void    setRightFooter (char *footer) - Specifies the string to
*            be displayed as the window's right footer.
*
*            char   *getLeftFooter () - Gets the left footer of the window.
*	     The string returned is newly allocated and must be freed by
*	     the caller.
*
*            char   *getRightFooter () - Gets the right footer of the window.
*	     The string returned is newly allocated and must be freed by
*	     the caller.
*
*            void    setFit (Dimension fit) - Depending on the value of the 
*            Dimension argument (HEIGHT or WIDTH), specifies that this window 
*            should fit to size around the Displays it contains in the 
*            specified Dimension.  Default: Don't fit the window.
*
*    virtual void    setDoneHandler (UICHandlerFunction) - Sets the function to
*    execute when the user attempts to unpin or DISMISS a PopUpWindow or QUIT 
*    a BaseWindow.  Default: Not set.
*
*            void    addChildWindow (UIWindow &window) - Specifies that the 
*            argument is a child window of the window.
*
*            void    addDisplay (UIDisplay &display) - Adds the UIDisplay
*            object to the list of display objects contained in this window. 
*
*    PROTECTED MEMBER FUNCTIONS:
*
*    void useDefaultDoneHandler (Boolean) - Specifies if the default done
*    handler should be used.
*
*    virtual void defaultDoneHandler () - The default done handler.
*
*    static void eventProc (Xv_window, Event *) - Internal wrapper
*           around the XView event proc for objects of classes derived
*           from UIWindow.
*
*    PRIVATE MEMBER FUNCTIONS:
*
*    void           setUIWindowDefaults () - Sets all the defaults values that 
*    have been previously defined.  This function is mainly used by the 
*    constructor to initialize a newly created UIWindow object.
*
*C-*/
#ifndef _UIWINDOW_H
#define _UIWINDOW_H

// Includes
#include "UIObject.h"
#include "GenericList.h"

#include <sys/param.h>
#include <xview/rect.h>
#include <xview/win_input.h>
#include <pixrect/pixrect.h>
#include <pixrect/pixfont.h>
#include <xview/frame.h>

// Class Declarations
class UIDisplay;

// Class Definition
class UIWindow : public UIObject {
public:
    		 	 UIWindow ();
    		 	 UIWindow (char *);
   			~UIWindow ();

    void		 setResizable (Boolean flag)
			 { resizable = flag;
			   if (owObject)
			     xv_set (owObject,
				     FRAME_SHOW_RESIZE_CORNER, resizable,
				     NULL); }

    void	 	 setDisplayFooter (Boolean footer)
			 { displayFooter = footer;
			   if (owObject)
			     xv_set (owObject,
				     FRAME_SHOW_FOOTER, displayFooter,
				     NULL); }

    void                 setLeftFooter (char *footer)
			 { leftFooter = strcpy (new char [strlen (footer) + 1],
						footer);
			   if (owObject)
			     xv_set (owObject,
				     FRAME_LEFT_FOOTER, leftFooter,
				     NULL); }

    void                 setRightFooter (char *footer)
			 { rightFooter =
			     strcpy (new char [strlen (footer) + 1], footer);
			   if (owObject)
			     xv_set (owObject,
				     FRAME_RIGHT_FOOTER, rightFooter,
				     NULL); }

    char		*getLeftFooter ()
			 { return (leftFooter ?
				   strcpy (new char [strlen (leftFooter) + 1],
					   leftFooter) :
				   (char *) NULL); } 

    char		*getRightFooter ()
			 { return (rightFooter ?
				   strcpy (new char [strlen (rightFooter) + 1],
					   rightFooter) :
				   (char *) NULL); } 

    void                 setFit (Dimension fit)
			 { if (fit == HEIGHT) {
			     fitHeight = TRUE;
			     if (owObject)
			       window_fit_height (owObject);
			   }
			   else {
			     fitWidth = TRUE;
			     if (owObject)
			       window_fit_width (owObject);
			   }
			 }

    virtual void	 setDoneHandler (UICHandlerFunction);

    void                 addChildWindow (UIWindow &window)
      { 
	childWindows.addItem (&window);
	
	int index = peerWindows->findItem (&window);
	if (index > -1)
	  (void) peerWindows->removeItem (index);
	
	window.parentWindow = this;
      }

    void 	 	 addDisplay (UIDisplay &display)
      {
	displayList.addItem ((Generic *) (&display));

	if (owObject)
	  ((UIObject *) &display)->createObject (this);
      }

protected:
    GenericList		 displayList;
    Boolean		 resizable;
    Boolean	 	 displayFooter;
    UICHandlerFunction	 doneHandler;
    char                *leftFooter;
    char   		*rightFooter;
    Boolean              fitHeight;
    Boolean              fitWidth;
    GenericList          childWindows;
    static GenericList  *peerWindows;
    UIWindow		*parentWindow;
    Boolean		 useDefaultDoneHandlerFlag;

    void useDefaultDoneHandler (Boolean flag)
      { useDefaultDoneHandlerFlag = flag; }

    virtual void defaultDoneHandler () {}
    
    static  void eventProc (Xv_window, Event *);

private:
    void setUIWindowDefaults ();
};
 
#endif /* _UIWINDOW_H */
